/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */


/*!
    \file   scas_log.h
    \brief  Logging API for the SCAS plugins and SCAS itself
    \author Oscar Koeroo for EGEE

    This header contains the declarations of the SCAS logging functions
    The SCAS plugins can use this API to write output to the SCAS logging
    devices.
    -# scas_log(): Log to SCAS logging devices.
    -# scas_log_debug(): Produce debugging output.
    \ingroup APIforLcmapsPlugins
*/

#ifndef SCAS_LOG_H
#define SCAS_LOG_H

/******************************************************************************
                             Include header files
******************************************************************************/
#include <stdio.h>
#include <syslog.h>

/******************************************************************************
 *                            Module definition
 *****************************************************************************/
#define MAX_LOG_BUFFER_SIZE 2048 /*!< Maximum logging buffer size, length of log
                                      may not exceed this number \internal */

#define DO_USRLOG           ((unsigned short)0x0001) /*!< flag to indicate that
                                      user logging has to be done \internal */
#define DO_SYSLOG           ((unsigned short)0x0002) /*!< flag to indicate that
                                      syslogging has to be done \internal */

#define DATETIMELEN         ((int) 20) /*!< Length of date time part in the log
                                            string \internal */

/******************************************************************************
Function:       scas_log()
Description:    Log information to file and or syslog
Parameters:
                prty:    syslog priority (if 0 don't syslog)
                fmt:     string format
Returns:        0 succes
                1 failure
******************************************************************************/
extern int scas_log(
        int prty,
        char * fmt,
        ...
);

/******************************************************************************
Function:       scas_log_debug()
Description:    Print debugging information
Parameters:
                debug_lvl: debugging level
                fmt:       string format
Returns:        0 succes
                1 failure
******************************************************************************/
extern int scas_log_debug(
        int debug_lvl,
        char * fmt,
        ...
);

/******************************************************************************
Function:       scas_log_time()
Description:    Log information to file and or syslog with a timestamp
Parameters:
                prty:    syslog priority (if 0 don't syslog)
                fmt:     string format
Returns:        0 succes
                1 failure
******************************************************************************/
extern int scas_log_time(
        int prty, 
        char * fmt, 
        ...
);

/******************************************************************************
Function:       scas_log_a_string()
Description:    Log a string according to the passed format to file and or syslog
Parameters:
                prty:       syslog priority (if 0 don't syslog)
                fmt:        string format
                the_string: the string
Returns:        0 succes
                1 failure
******************************************************************************/
extern int scas_log_a_string(
        int prty,
        char * fmt,
        char * the_string
);

/******************************************************************************
Function:       scas_log_a_string_debug()
Description:    Print debugging information
Parameters:
                debug_lvl:  debugging level
                fmt:        string format
                the_string: the string
Returns:        0 succes
                1 failure
******************************************************************************/
extern int scas_log_a_string_debug(
        int debug_lvl,
        char * fmt,
        char * the_string
);



/******************************************************************************
Function:       scas_log_set_time_indicator()
Description:    Set the log time string to time of its invocation for the 
                continuously written log lines
Parameters:
Returns:        0 succes
                1 failure
******************************************************************************/
int scas_log_set_time_indicator (void);


/******************************************************************************
Function:       scas_get_log_file_pointer()
Description:    Get the opened file pointer
Parameters:
Returns:        0 succes
                1 failure
******************************************************************************/
extern FILE * scas_get_log_file_pointer(void);


/******************************************************************************
Function:       scas_get_log_type (void);
Description:    Get the log type identification number
Parameters:
Returns:        0 succes
                1 failure
******************************************************************************/
extern unsigned short scas_get_log_type (void);


/******************************************************************************
Function:       scas_get_log_path (void);
Description:    Get the configured and used path to the log file as string
Parameters:
Returns:        !NULL succes
                NULL  failure or not set due to syslog logging only
******************************************************************************/
extern char * scas_get_log_path (void);

#endif /* SCAS_LOG_H */

/******************************************************************************
CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/scas/interface/scas_log.h,v $
    $Date: 2010-04-29 11:10:59 $
    $Revision: 1.7 $
    $Author: okoeroo $
******************************************************************************/
