/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */



/* network functions */
/*
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <unistd.h>
*/

#include <errno.h>
#include "net_common.h"
#include "scas_log.h"


/****************************************************
Function: setup_master_socket
Description:
          Setup Master Socket with all the socket options and such.

Variables:
          int * master_socket : master socket descriptor
          int   port          : port number to listen on
Return:
    0 : Good
   !0 : Not good
****************************************************/
int setup_master_socket (int * master_socket, int port)
{
    if ((port > 65535) || (port < 0))
    {
        scas_log (1, "%s: Failure in setting up listening socket: TCP port number out of range (0-65535) with value %d\n", __func__, port);
        return 1;
    }

    if ((*master_socket = createAndSetUpATCPServerSocket((unsigned short) port, 512)) < 0)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/* Listening on both IPv4 and IPv6 sockets */
#if 0
{
    int opt = 1;
    struct sockaddr_in address;

    /* create the master socket and check it worked */
    if ((*master_socket = socket(AF_INET,SOCK_STREAM,0)) < 0)
    {
        scas_log (1, "Failure in create master_socket fail\n");
        return 1;
    }

    /* set master socket to allow multiple connections */
    if (setsockopt(*master_socket, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(opt)) < 0)
    {
        scas_log (1, "Failure in setsockopt\n");
        return 2;
    }

    /* type of socket created */
    address.sin_family      = AF_INET;
    address.sin_addr.s_addr = INADDR_ANY;
    address.sin_port        = htons(port);

    /* bind master socket to port number */
    if (bind(*master_socket, (struct sockaddr *)&address, sizeof(address)) < 0)
    {
        scas_log (1, "Failure in bind\n");
        return 3;
    }


    /* 
     * Try to specify maximum of 512 pending connections,
     * for  the master socket 
     * This number was derived from Apache examples
     */
    if (listen(*master_socket, 512) < 0)
    {
        scas_log (1, "Failure in setting socket to listen\n");
        return 4;
    }

    return 0;
}
#endif


/***************************************************
Function:
    createAndSetUpATCPServerSocket
Description:
    This function is a direct descendant of the examples by the 
    EGEE-III SA2 group.
    The function will return with -1 or with an active bound and listening master socket
    which will listen for IPv4 and IPv6 clients to connect.
 
Parameters:
    port                Sets the TCP portnumber on which the service will listen
                        The rang is from 0 to 65535
    max_connections     Sets the amount of listener sockets for the process to preload
                        and which can be dispatched to accept new connections.
***************************************************/
int createAndSetUpATCPServerSocket(unsigned short port, int max_connections)
{
    int server_socket = 0;
    struct addrinfo hints;
    struct addrinfo *address_list = NULL;
    struct addrinfo *paddress = NULL;

    int on = 1, off = 0;
    /* char ip[IP_MAX_CHARS]; */
    char * ip = NULL;
    char port_string[PORT_MAX_DIGITS];

    server_socket = -1;

    // getaddrinfo parameters
    memset(&hints, 0, sizeof(hints));   // init
    hints.ai_flags |= AI_PASSIVE;       // server behaviour: accept connection on any network address
    hints.ai_family = AF_UNSPEC;        // find both ipv4 and ipv6 addresses
    hints.ai_socktype = SOCK_STREAM;    // type of socket

    sprintf (port_string, "%d", port);
    getaddrinfo(NULL, port_string, &hints, &address_list);

    paddress = address_list;
    while (paddress)
    {
        // create the socket
        server_socket = socket(paddress->ai_family, paddress->ai_socktype, paddress->ai_protocol);
        if (server_socket == -1)
        {
            paddress = paddress->ai_next;
            continue; // try next address
        }

        // set SO_REUSEADDR option
        setsockopt(server_socket, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on));

        if (paddress->ai_family == AF_INET6)
        {   // we must unset IPV6_V6ONLY in order to allow IPv4 clients
            // to connect to the IPv6 socket
            setsockopt(server_socket, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&off, sizeof(off));
        }

        // bind and listen
        if (    (bind(server_socket, paddress->ai_addr, paddress->ai_addrlen)   == -1) ||
            (listen(server_socket, max_connections)             == -1))
        {
            close(server_socket); server_socket = -1;
            paddress = paddress->ai_next;
            continue; // try next address
        }

        // get the ip string, display it with the port number 
        getIPString(ip, IP_MAX_CHARS, paddress->ai_addr);
        net_addr2host (&ip, IP_MAX_CHARS, paddress->ai_addr);
        scas_log (1, "Server socket now listening on %s port %d\n", ip, port);

        break; // break if no error
    }

    // free the list
    freeaddrinfo(address_list);

    // return the server socket
    return server_socket;
}




/****************************************************
Function: accept_new_client_socket
Description:
          Accepts new client socket from a master socket and returns that with the 
          DNS canonical name of the connected remote host

Variables:
          int master_socket   : master socket descriptor
          int * client_socket : return pointer to the client_socket descriptor
          char ** remote_host : DNS canonical name string of the remote host 
                                connecting to the master socket
Return:
    0 : Good
   !0 : Not good
****************************************************/
#if 0
/* Old way, replaced by the IPv6 compliant one */
int accept_new_client_socket (int                  master_socket, 
                              int                * client_socket, 
                              struct sockaddr_in * addr,
                              socklen_t          * addr_len,
                              char              ** remote_host)
{
    int new_socket = -1;

    scas_log_debug (5, "Accepting new client connection\n");


    /* Open the new socket as 'new_socket' */
    if ((new_socket = accept(master_socket, (struct sockaddr *) addr, addr_len)) < 0)
    {
        scas_log (3, "Accept of client socket fail!\n");
        return 1;
    }


    /* Get remote_host string from opened connection */
    *remote_host = (char *) malloc (sizeof(char) * 255);
    bzero (*remote_host, sizeof(char) * 255);

    addr2host (remote_host, 255, (struct sockaddr_in) *addr);

    scas_log_debug (4, "Service: Accepted connection from remote host: %s\n", *remote_host);

    /* add new socket to list of sockets */
    *client_socket = new_socket;

    return 0;
}
#endif


/****************************************************
Function: accept_new_client_socket
Description:
          Accepts new client socket from a master socket and returns that with the 
          DNS canonical name of the connected remote host

Variables:
          int master_socket   : master socket descriptor
          int * client_socket : return pointer to the client_socket descriptor
          char ** remote_host : DNS canonical name string of the remote host 
                                connecting to the master socket
Return:
    0 : Good
   !0 : Not good
****************************************************/
/* int manageClientConnectedOnServerSocket(int server_socket) */
int accept_new_client_socket (int                       server_socket, 
                              int                     * client_socket, 
                              struct sockaddr_storage * addr,
                              socklen_t               * addr_len,
                              char                   ** remote_host)
{
    struct sockaddr_storage sock_addr;
    /* struct sockaddr *client_socket_address = (struct sockaddr *)&sock_addr; */
    /* unsigned int client_socket_address_len = sizeof(sock_addr); */
    struct sockaddr *client_socket_address = (struct sockaddr *)&sock_addr;
    unsigned int client_socket_address_len = sizeof(sock_addr);

    /* accept connection */
    *client_socket = accept(server_socket, client_socket_address, &client_socket_address_len);

    if (*client_socket < 0)
    {
        scas_log (5, "%s: Client socket not accepted: %s\n", __func__, strerror(errno));
        return 1;
    }

    /* notify client connection */
    *remote_host = (char *) calloc (1, HOSTNAME_MAX_CHARS);
    net_addr2host(remote_host, HOSTNAME_MAX_CHARS, client_socket_address);


    return 0;
}




